class TooManyMatchingGroupsError(Exception):
    pass


class NoMatchingGroupError(Exception):
    pass


class UnableToFindApplicationError(Exception):
    pass


class UnableToFindUserError(Exception):
    pass
