from ms_graph_client.services.applications import Applications as ApplicationsCrud
from typing import Any

class Applications:
    _crud_client: ApplicationsCrud
    def __init__(self, crud_client: ApplicationsCrud) -> None: ...
    def get_by_application_name(self, app_name: str) -> Any: ...
    def get_service_principal_by_app_id(self, app_id: str) -> Any: ...
    def exists_by_obj_id(self, id: str) -> bool: ...
    def get_by_object_id(self, id: str) -> Any: ...
