from ms_graph_client import GraphAPIConfig

class Generator:
    config: GraphAPIConfig

    def __init__(self, config: GraphAPIConfig): ...
    def user_url(self, user_obj_id: str) -> str: ...
    def service_principal_url(self, app_service_principal_id: str) -> str:...
