from ms_graph_client.services.groups import Groups as GroupsCrud
from typing import Any

class Groups:
    _crud_client: GroupsCrud
    def __init__(self, crud_client: GroupsCrud) -> None: ...
    def is_group_assigned_to_app(self, app_service_principal_id: str, group_id: str) -> bool: ...
    def get_by_name(self, group_name: str) -> Any: ...
    def exists_by_name(self, group_name: str) -> bool: ...
    def list_group_members(self, group_id: str) -> list[Any]: ...
    def is_member_of_group(self, group_id: str, object_id: str) -> bool: ...
